package com.weface.component;

import com.gexin.rp.sdk.base.IBatch;
import com.gexin.rp.sdk.base.IIGtPush;
import com.gexin.rp.sdk.base.IPushResult;
import com.gexin.rp.sdk.base.ITemplate;
import com.gexin.rp.sdk.base.impl.SingleMessage;
import com.gexin.rp.sdk.base.impl.Target;
import com.gexin.rp.sdk.base.payload.APNPayload;
import com.gexin.rp.sdk.base.payload.MultiMedia;
import com.gexin.rp.sdk.exceptions.RequestException;
import com.gexin.rp.sdk.http.Constants;
import com.gexin.rp.sdk.http.IGtPush;
import com.gexin.rp.sdk.template.NotificationTemplate;
import com.gexin.rp.sdk.template.style.Style0;
import com.gexin.rp.sdk.template.style.Style6;
import com.weface.config.GeTuiApp;
import com.weface.config.GeTuiConfig;
import com.weface.dto.MsgDTO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

@Component
@Slf4j
public class GeTuiSDK {

    @Resource
    private GeTuiApp geTuiApp;

    private GeTuiConfig getConfig(String fromApp){
        return geTuiApp.getApps().get(fromApp);
    }

    public IPushResult pushSingle(String alias, String title, String body) {
        // 设置后，根据别名推送，会返回每个cid的推送结果
        System.setProperty(Constants.GEXIN_PUSH_SINGLE_ALIAS_DETAIL, "true");
        String fromApp = alias.split("_")[0];
        GeTuiConfig config = getConfig(fromApp);
        String appId = config.getAppId();
        String appKey = config.getAppkey();
        IGtPush push = new IGtPush(config.getBaseUrl(), appKey, config.getMastersecret());
        NotificationTemplate template = getNotificationTemplate(title, body,appId,appKey);
        SingleMessage message = new SingleMessage();
        message.setOffline(true);
        // 离线有效时间，单位为毫秒
        message.setOfflineExpireTime(24 * 3600 * 1000);
        message.setData(template);
        // 可选，1为wifi，0为不限制网络环境。根据手机处于的网络情况，决定是否下发
        message.setPushNetWorkType(0);
        // 厂商通道下发策略
        message.setStrategyJson("{\"default\":4,\"ios\":4,\"st\":4}");
        Target target = new Target();
        target.setAppId(appId);
        //target.setClientId(CID);
        target.setAlias(alias);
        IPushResult ret = null;
        try {
            ret = push.pushMessageToSingle(message, target);
            log.info(alias+"->返回:{}", ret.getResponse().toString());
        } catch (RequestException e) {
            e.printStackTrace();
            log.debug(e.getMessage());
            ret = push.pushMessageToSingle(message, target, e.getRequestId());
        }
        return ret;
    }

    public NotificationTemplate getNotificationTemplate(String title, String body,String appId,String appKey) {
        NotificationTemplate template = new NotificationTemplate();
        // 设置APPID与APPKEY
        template.setAppId(appId);
        template.setAppkey(appKey);

        Style0 style = new Style0();
        Style6 style6 = new Style6();
        style6.setText(body);
        style6.setBigStyle2(body);
        style6.setLogo("icon.png");
        style6.setChannelLevel(4);
        style6.setTitle(title);
        style6.setRing(true);
        style6.setVibrate(true);
        style6.setClearable(true);
        /* // 设置通知栏标题与内容
        style.setTitle(title);
        style.setText(body);
        // 配置通知栏图标
        style.setLogo("icon.png");
        // 配置通知栏网络图标
        style.setLogoUrl("");
        // 设置通知是否响铃，震动，或者可清除
        style.setRing(true);
        style.setVibrate(true);
        style.setClearable(true);
        //style.setChannel("通知渠道id");
        //style.setChannelName("通知渠道名称");
        style.setChannelLevel(3); //设置通知渠道重要性*/
        template.setStyle(style6);

        template.setTransmissionType(1);  // 透传消息接受方式设置，1：立即启动APP，2：客户端收到消息后需要自行处理
        template.setTransmissionContent("请输入您要透传的内容");

        template.setAPNInfo(getAPNPayload(body)); //详见【推送模板说明】iOS通知样式设置
        return template;
    }


    private APNPayload getAPNPayload(String body) {
        APNPayload payload = new APNPayload();
        //在已有数字基础上加1显示，设置为-1时，在已有数字上减1显示，设置为数字时，显示指定数字
        payload.setAutoBadge("+1");
        payload.setContentAvailable(0);
        //ios 12.0 以上可以使用 Dictionary 类型的 sound
        payload.setSound("default");
        payload.setCategory("$由客户端定义");
        payload.addCustomMsg("由客户自定义消息key", "由客户自定义消息value");

        //简单模式APNPayload.SimpleMsg
        payload.setAlertMsg(new APNPayload.SimpleAlertMsg(body));
        //payload.setAlertMsg(getDictionaryAlertMsg());  //字典模式使用APNPayload.DictionaryAlertMsg

        //设置语音播报类型，int类型，0.不可用 1.播放body 2.播放自定义文本
        payload.setVoicePlayType(2);
        //设置语音播报内容，String类型，非必须参数，用户自定义播放内容，仅在voicePlayMessage=2时生效
        //注：当"定义类型"=2, "定义内容"为空时则忽略不播放
        payload.setVoicePlayMessage("定义内容");

        // 添加多媒体资源
        payload.addMultiMedia(new MultiMedia().setResType(MultiMedia.MediaType.pic)
                .setResUrl("资源文件地址")
                .setOnlyWifi(true));

        return payload;
    }


    public void batchPush(List<MsgDTO> objs,String fromApp) throws Exception {
        GeTuiConfig config = getConfig(fromApp);
        IIGtPush push = new IGtPush(config.getBaseUrl(), config.getAppkey(), config.getMastersecret());
        IBatch batch = push.getBatch();
        for (MsgDTO msg : objs) {
            constructClientNotifyMsg(msg.getFromApp() + "_" + msg.getUserId(), msg.getTitle(), msg.getBody(), batch,config.getAppId(),config.getAppkey());
        }
        batch.submit();
    }

    public void constructClientNotifyMsg(String alia, String title, String body, IBatch batch,String appId,String appKey) throws Exception {
        SingleMessage message = new SingleMessage();
        ITemplate template = notificationTemplateDemo(title, body,appId,appKey);

        //template.setAPNInfo(getAPNPayload()); //详见【推送模板说明】iOS通知样式设置

        message.setData(template);
        message.setOffline(true);
        message.setOfflineExpireTime(360 * 1000);
        // 厂商通道下发策略
        message.setStrategyJson("{\"default\":4,\"ios\":4,\"st\":4}");

        // 设置推送目标，填入appid和clientId
        Target target = new Target();
        target.setAppId(appId);
        target.setAlias(alia);
        batch.add(message, target);
    }


    public NotificationTemplate notificationTemplateDemo(String title, String body,String appId,String appKey) {
        NotificationTemplate template = new NotificationTemplate();
        // 设置APPID与APPKEY
        template.setAppId(appId);
        template.setAppkey(appKey);

        Style0 style = new Style0();
        // 设置通知栏标题与内容
        style.setTitle(title);
        style.setText(body);
        // 配置通知栏图标
        style.setLogo("icon.png");
        // 配置通知栏网络图标
        style.setLogoUrl("");
        // 设置通知是否响铃，震动，或者可清除
        style.setRing(true);
        style.setVibrate(true);
        style.setClearable(true);
        style.setChannel("通知渠道id");
        style.setChannelName("通知渠道名称");
        style.setChannelLevel(3); //设置通知渠道重要性
        template.setStyle(style);

        // 透传消息接受方式设置，1：立即启动APP，2：客户端收到消息后需要自行处理
        template.setTransmissionType(2);
        template.setTransmissionContent("请输入您要透传的内容");

        template.setAPNInfo(getAPNPayload(body)); //详见【推送模板说明】iOS通知样式设置
        return template;
    }


}
